Class: AgentHarness::Providers::Codex
Overview
OpenAI Codex CLI provider
Provides integration with the OpenAI Codex CLI tool.
Constant Summary
collapse
- SUPPORTED_CLI_VERSION =
"0.116.0"
- SUPPORTED_CLI_REQUIREMENT =
Gem::Requirement.new(">= #{SUPPORTED_CLI_VERSION}", "< 0.117.0").freeze
- OAUTH_REFRESH_FAILURE_PATTERNS =
[
/refresh_token_reused/i,
/failed to refresh token\b.*\b401\b/im,
/failed to refresh token\b.*unauthorized/im,
/failed to refresh token\b.*\binvalid_client\b/im,
/failed to refresh token\b.*\binvalid_grant\b/im,
/failed to refresh token\b.*invalid.*refresh.*token/im,
/failed to refresh token\b.*refresh.*token.*invalid/im,
/your access token could not be refreshed because\b.*\b401\b/im,
/your access token could not be refreshed because\b.*unauthorized/im,
/your access token could not be refreshed because\b.*\binvalid_client\b/im,
/your access token could not be refreshed because\b.*\binvalid_grant\b/im,
/your access token could not be refreshed because\b.*invalid.*refresh.*token/im,
/your access token could not be refreshed because\b.*refresh.*token.*invalid/im,
/your access token could not be refreshed because\s+your refresh token .*already (?:been )?used/im,
/refresh token .*already (?:been )?used/im
].freeze
- OAUTH_REFRESH_TRANSIENT_PATTERNS =
[
/your access token could not be refreshed because\s+(?:the\s+)?auth(?:entication)? service(?:\s+(?:is|was))?\s+(?:temporarily\s+)?unavailable/im,
/your access token could not be refreshed because .*connection.*error/im,
/failed to refresh token\b.*connection.*error/im,
/failed to refresh token\b.*service(?:\s+(?:is|was))?\s+(?:temporarily\s+)?unavailable/im
].freeze
Constants inherited
from Base
Base::COMMON_ERROR_PATTERNS, Base::DEFAULT_SMOKE_TEST_CONTRACT
Instance Attribute Summary
Attributes inherited from Base
#config, #executor, #logger
Class Method Summary
collapse
Instance Method Summary
collapse
#parse_rate_limit_reset
Methods inherited from Base
#configure, #initialize, #parse_rate_limit_reset, #parse_test_error, #sandboxed_environment?, #send_message
Methods included from Adapter
#auth_type, #build_mcp_flags, #fetch_mcp_servers, included, metadata_package_name, #noisy_error_patterns, normalize_metadata_installation, normalize_metadata_source_type, normalize_metadata_version_requirement, #parse_rate_limit_reset, #send_message, #smoke_test, #smoke_test_contract, #supported_mcp_transports, #supports_dangerous_mode?, #supports_mcp?, #supports_text_mode?, #supports_token_counting?, #supports_tool_control?, #validate_mcp_servers!
Class Method Details
.binary_name ⇒ Object
44
45
46
|
# File 'lib/agent_harness/providers/codex.rb', line 44
def binary_name
"codex"
end
|
.discover_models ⇒ Object
82
83
84
85
86
87
88
|
# File 'lib/agent_harness/providers/codex.rb', line 82
def discover_models
return [] unless available?
[
{name: "codex", family: "codex", tier: "standard", provider: "codex"}
]
end
|
.firewall_requirements ⇒ Object
62
63
64
65
66
67
68
69
70
|
# File 'lib/agent_harness/providers/codex.rb', line 62
def firewall_requirements
{
domains: [
"api.openai.com",
"openai.com"
],
ip_ranges: []
}
end
|
.installation_contract(version: SUPPORTED_CLI_VERSION) ⇒ Object
90
91
92
93
94
95
96
97
98
99
100
101
102
103
104
105
106
107
108
109
110
111
112
113
114
115
116
117
118
119
120
121
122
123
124
125
126
127
128
129
130
131
132
133
134
135
136
137
|
# File 'lib/agent_harness/providers/codex.rb', line 90
def installation_contract(version: SUPPORTED_CLI_VERSION)
version = version.strip if version.respond_to?(:strip)
unless version.is_a?(String) && !version.empty?
raise ArgumentError,
"Unsupported Codex CLI version #{version.inspect}; " \
"supported versions must satisfy #{SUPPORTED_CLI_REQUIREMENT}"
end
parsed_version = begin
Gem::Version.new(version)
rescue ArgumentError
raise ArgumentError,
"Unsupported Codex CLI version #{version.inspect}; " \
"supported versions must satisfy #{SUPPORTED_CLI_REQUIREMENT}"
end
unless SUPPORTED_CLI_REQUIREMENT.satisfied_by?(parsed_version)
raise ArgumentError,
"Unsupported Codex CLI version #{version.inspect}; " \
"supported versions must satisfy #{SUPPORTED_CLI_REQUIREMENT}"
end
default_package = "@openai/codex@#{version}".freeze
install_command_prefix = ["npm", "install", "-g", "--ignore-scripts"].freeze
install_command = (install_command_prefix + [default_package]).freeze
supported_versions = [version].freeze
version_requirement = SUPPORTED_CLI_REQUIREMENT.requirements
.map { |op, ver| "#{op} #{ver}".freeze }
.freeze
contract = {
source: :npm,
package: default_package,
package_name: "@openai/codex",
version: version,
version_requirement: version_requirement,
binary_name: binary_name,
install_command_prefix: install_command_prefix,
install_command: install_command,
supported_versions: supported_versions
}
contract.each_value do |value|
value.freeze if value.is_a?(String)
end
contract.freeze
end
|
.instruction_file_paths ⇒ Object
72
73
74
75
76
77
78
79
80
|
# File 'lib/agent_harness/providers/codex.rb', line 72
def instruction_file_paths
[
{
path: "AGENTS.md",
description: "OpenAI Codex agent instructions",
symlink: false
}
]
end
|
.parse_cli_jsonl_transcript(raw_output, max_events: nil) ⇒ Object
143
144
145
146
147
148
149
|
# File 'lib/agent_harness/providers/codex.rb', line 143
def parse_cli_jsonl_transcript(raw_output, max_events: nil)
return new.send(:parse_jsonl_output, "") if max_events && max_events <= 0
output = max_events ? tail_nonempty_lines(raw_output, limit: max_events).join("\n") : raw_output
new.send(:parse_jsonl_output, output)
end
|
53
54
55
56
57
58
59
60
|
# File 'lib/agent_harness/providers/codex.rb', line 53
def provider_metadata_overrides
{
auth: {
service: :openai,
api_family: :openai
}
}
end
|
.provider_name ⇒ Object
40
41
42
|
# File 'lib/agent_harness/providers/codex.rb', line 40
def provider_name
:codex
end
|
.smoke_test_contract ⇒ Object
Instance Method Details
#api_key_env_var_names ⇒ Object
194
|
# File 'lib/agent_harness/providers/codex.rb', line 194
def api_key_env_var_names = ["OPENAI_API_KEY"]
|
#api_key_unset_vars ⇒ Object
196
|
# File 'lib/agent_harness/providers/codex.rb', line 196
def api_key_unset_vars = ["OPENAI_BASE_URL", "OPENAI_HEADER_X_AGENT_RUN_ID", "OPENAI_HEADER_X_PROXY_TOKEN"]
|
#auth_lock_config ⇒ Object
364
365
366
|
# File 'lib/agent_harness/providers/codex.rb', line 364
def auth_lock_config
{path: "/tmp/codex-auth.lock", timeout: 30}
end
|
#auth_status ⇒ Object
290
291
292
293
294
295
296
297
298
299
300
301
302
303
304
305
306
307
308
309
310
311
312
313
314
315
|
# File 'lib/agent_harness/providers/codex.rb', line 290
def auth_status
api_key = ENV["OPENAI_API_KEY"]
if api_key && !api_key.strip.empty?
if api_key.strip.start_with?("sk-")
return {valid: true, expires_at: nil, error: nil, auth_method: :api_key}
else
return {valid: false, expires_at: nil, error: "OPENAI_API_KEY is set but does not appear to be a valid OpenAI API key", auth_method: nil}
end
end
credentials = read_codex_credentials
if credentials
key = credentials["api_key"] || credentials["apiKey"] || credentials["OPENAI_API_KEY"]
if key.is_a?(String) && !key.strip.empty?
if key.strip.start_with?("sk-")
return {valid: true, expires_at: nil, error: nil, auth_method: :config_file}
else
return {valid: false, expires_at: nil, error: "Config file API key is set but does not appear to be a valid OpenAI API key", auth_method: nil}
end
end
end
{valid: false, expires_at: nil, error: "No OpenAI API key found. Set OPENAI_API_KEY or configure in #{codex_config_path}", auth_method: nil}
rescue IOError, JSON::ParserError => e
{valid: false, expires_at: nil, error: e.message, auth_method: nil}
end
|
#capabilities ⇒ Object
182
183
184
185
186
187
188
189
190
191
192
|
# File 'lib/agent_harness/providers/codex.rb', line 182
def capabilities
{
streaming: false,
file_upload: false,
vision: false,
tool_use: true,
json_mode: false,
mcp: false,
dangerous_mode: true
}
end
|
#cli_env_overrides ⇒ Object
200
|
# File 'lib/agent_harness/providers/codex.rb', line 200
def cli_env_overrides = {"PAID_CODEX_SUBSCRIPTION_AUTH" => "1"}
|
#config_file_content(options = {}) ⇒ Object
346
347
348
349
350
351
352
353
354
|
# File 'lib/agent_harness/providers/codex.rb', line 346
def config_file_content(options = {})
<<~TOML
[chatgpt]
model_provider = "#{escape_toml_string(options[:model_provider])}"
base_url = "#{escape_toml_string(options[:base_url])}"
env_key = "#{escape_toml_string(options[:env_key])}"
wire_api = "#{escape_toml_string(options[:wire_api])}"
TOML
end
|
#configuration_schema ⇒ Object
174
175
176
177
178
179
180
|
# File 'lib/agent_harness/providers/codex.rb', line 174
def configuration_schema
{
fields: [],
auth_modes: [:api_key],
openai_compatible: true
}
end
|
#dangerous_mode_flags ⇒ Object
206
207
208
|
# File 'lib/agent_harness/providers/codex.rb', line 206
def dangerous_mode_flags
["--full-auto"]
end
|
#display_name ⇒ Object
170
171
172
|
# File 'lib/agent_harness/providers/codex.rb', line 170
def display_name
"OpenAI Codex CLI"
end
|
#error_classification_patterns ⇒ Object
265
266
267
268
269
270
271
272
273
274
275
276
277
278
279
280
|
# File 'lib/agent_harness/providers/codex.rb', line 265
def error_classification_patterns
super.merge(
auth_expired: [
/refresh_token_reused/i,
/refresh token has already been used/i,
/Please log out and sign in again/i,
/authentication_error/i,
/invalid_grant/i,
/Token is expired or invalid/i
],
abort: [
/free tier limit reached/i,
/please upgrade to a paid plan/i
]
)
end
|
#error_patterns ⇒ Object
242
243
244
245
246
247
248
249
250
251
252
253
254
255
256
257
258
259
260
261
262
263
|
# File 'lib/agent_harness/providers/codex.rb', line 242
def error_patterns
{
rate_limited: COMMON_ERROR_PATTERNS[:rate_limited],
timeout: [
/your access token could not be refreshed.*(?:timeout|timed.?out)/im,
/failed to refresh token\b.*(?:timeout|timed.?out)/im
],
transient: COMMON_ERROR_PATTERNS[:transient] + [
/connection.*reset/i
] + OAUTH_REFRESH_TRANSIENT_PATTERNS,
auth_expired: COMMON_ERROR_PATTERNS[:auth_expired] + [
/\b401\b/,
/incorrect.*api.*key/i
] + OAUTH_REFRESH_FAILURE_PATTERNS,
quota_exceeded: COMMON_ERROR_PATTERNS[:quota_exceeded],
sandbox_failure: [
/bwrap.*no permissions/i,
/no permissions to create a new namespace/i,
/unprivileged.*namespace/i
]
}
end
|
#execution_semantics ⇒ Object
220
221
222
223
224
225
226
227
228
229
230
231
|
# File 'lib/agent_harness/providers/codex.rb', line 220
def execution_semantics
{
prompt_delivery: :arg,
output_format: :json,
sandbox_aware: true,
uses_subcommand: true,
non_interactive_flag: nil,
legitimate_exit_codes: [0],
stderr_is_diagnostic: true,
parses_rate_limit_reset: false
}
end
|
#health_status ⇒ Object
317
318
319
320
321
322
323
324
325
326
327
328
|
# File 'lib/agent_harness/providers/codex.rb', line 317
def health_status
unless self.class.available?
return {healthy: false, message: "Codex CLI not found in PATH. Install from https://github.com/openai/codex"}
end
auth = auth_status
unless auth[:valid]
return {healthy: false, message: auth[:error]}
end
{healthy: true, message: "Codex CLI available and authenticated"}
end
|
#name ⇒ Object
166
167
168
|
# File 'lib/agent_harness/providers/codex.rb', line 166
def name
"codex"
end
|
#notify_hook_content ⇒ Object
356
357
358
359
360
361
362
|
# File 'lib/agent_harness/providers/codex.rb', line 356
def notify_hook_content
<<~TOML
[notify]
# Paid notification hook
TOML
end
|
#session_flags(session_id) ⇒ Object
237
238
239
240
|
# File 'lib/agent_harness/providers/codex.rb', line 237
def session_flags(session_id)
return [] unless session_id && !session_id.empty?
["--session", session_id]
end
|
#subscription_unset_vars ⇒ Object
198
|
# File 'lib/agent_harness/providers/codex.rb', line 198
def subscription_unset_vars = ["OPENAI_API_KEY", "OPENAI_BASE_URL"] + api_key_unset_vars
|
#supports_sessions? ⇒ Boolean
233
234
235
|
# File 'lib/agent_harness/providers/codex.rb', line 233
def supports_sessions?
true
end
|
#test_command_overrides ⇒ Object
202
203
204
|
# File 'lib/agent_harness/providers/codex.rb', line 202
def test_command_overrides
["--skip-git-repo-check", "--output-last-message"]
end
|
#token_usage_from_api_response(body) ⇒ Object
210
211
212
213
214
215
216
217
218
|
# File 'lib/agent_harness/providers/codex.rb', line 210
def token_usage_from_api_response(body)
usage = body&.dig("usage")
return {} unless usage
{
input_tokens: usage["prompt_tokens"].to_i,
output_tokens: usage["completion_tokens"].to_i
}
end
|
#translate_error(message) ⇒ Object
282
283
284
285
286
287
288
|
# File 'lib/agent_harness/providers/codex.rb', line 282
def translate_error(message)
case message
when /refresh_token_reused/i then "Codex authentication expired. Please re-authenticate."
when /free tier limit/i then "Codex free tier limit reached."
else message
end
end
|
#validate_config ⇒ Object
330
331
332
333
334
335
336
337
338
339
340
341
342
343
344
|
# File 'lib/agent_harness/providers/codex.rb', line 330
def validate_config
errors = []
flags = @config.default_flags
unless flags.nil?
if flags.is_a?(Array)
invalid = flags.reject { |f| f.is_a?(String) }
errors << "default_flags contains non-string values" if invalid.any?
else
errors << "default_flags must be an array of strings"
end
end
{valid: errors.empty?, errors: errors}
end
|